import numpy as np
import matplotlib.pyplot as plt

##========================= IIR Filter Constraints ========================##
filter_length = 500                                             # Filter Length
delta_t = 0.1                                                   # Sampling Period

##=============================== Parameters ==============================##
dt = 0.001                                                      # Precision in Time Domain
t = np.arange(-100, 100, dt)                                    # Continuous Frequency

##===================== Desired Filter in Time Domain =====================##
h_a = np.sin(np.pi*t)/(np.pi*t)                                 # Desired Analog Filter in Time Domain


if filter_length > int(len(t)*dt/delta_t):
    print("Output IIR filter length is too long / Sampling period is too large")
    exit()

##================================ Step 1 =================================##
h_au = np.array([0j for _ in range(len(h_a))])
h_u = np.array([0j for _ in range(int(len(h_a)/int(delta_t/dt)))])
h_au[0] = h_a[0]*dt
h_u[0] = h_a[0]*dt

for i in range(1,len(h_au)):
    h_au[i] = h_au[i-1]+h_a[i]*dt                               # Unit-Step Response of the Desired Analog Filter

##================================ Step 2 =================================##
for i in range(len(h_u)):
    h_u[i] = h_au[i*int(delta_t/dt)]                            # Perform Sampling for h_au


##================================ Step 3 =================================##
h = np.array([0j for _ in range(len(h_u))])
h[0] = h_u[0]
for i in range(1, len(h_u)):
    h[i] = h_u[i]-h_u[i-1]                                      # Calculate h[n] from h_u[n]-h_u[n-1]


##===================== Output Filter in D.T Domain =======================##
center = np.argmax(abs(h))
origin = int(len(h)/2)
h = h[int(center-np.floor(0.5*filter_length+1)): int(center+np.floor(0.5*filter_length))]
n = np.arange(int(center-np.floor(0.5*filter_length+1))-origin, int(center+np.floor(0.5*filter_length))-origin, 1)



##================================ PLOT ===================================##
fig, ax = plt.subplots()
ax.plot(t,h_a, label="Ideal Filter in Time Domain")

plt.xlabel('Time (Continuous)', fontsize=18)
plt.ylabel('Impulse Response', fontsize=18)
plt.legend(fontsize=18)
plt.show()

plt.stem(n ,h)
plt.xlabel('Time (Discrete)', fontsize=18)
plt.ylabel('Impulse Response', fontsize=18)
plt.show()
